/* ***********************************************************
 * This file was automatically generated on 2012-10-07.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling stepper motors
	/// </summary>
	public class BrickStepper : Device 
	{
		private static byte FUNCTION_SET_MAX_VELOCITY = 1;
		private static byte FUNCTION_GET_MAX_VELOCITY = 2;
		private static byte FUNCTION_GET_CURRENT_VELOCITY = 3;
		private static byte FUNCTION_SET_SPEED_RAMPING = 4;
		private static byte FUNCTION_GET_SPEED_RAMPING = 5;
		private static byte FUNCTION_FULL_BRAKE = 6;
		private static byte FUNCTION_SET_CURRENT_POSITION = 7;
		private static byte FUNCTION_GET_CURRENT_POSITION = 8;
		private static byte FUNCTION_SET_TARGET_POSITION = 9;
		private static byte FUNCTION_GET_TARGET_POSITION = 10;
		private static byte FUNCTION_SET_STEPS = 11;
		private static byte FUNCTION_GET_STEPS = 12;
		private static byte FUNCTION_GET_REMAINING_STEPS = 13;
		private static byte FUNCTION_SET_STEP_MODE = 14;
		private static byte FUNCTION_GET_STEP_MODE = 15;
		private static byte FUNCTION_DRIVE_FORWARD = 16;
		private static byte FUNCTION_DRIVE_BACKWARD = 17;
		private static byte FUNCTION_STOP = 18;
		private static byte FUNCTION_GET_STACK_INPUT_VOLTAGE = 19;
		private static byte FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE = 20;
		private static byte FUNCTION_GET_CURRENT_CONSUMPTION = 21;
		private static byte FUNCTION_SET_MOTOR_CURRENT = 22;
		private static byte FUNCTION_GET_MOTOR_CURRENT = 23;
		private static byte FUNCTION_ENABLE = 24;
		private static byte FUNCTION_DISABLE = 25;
		private static byte FUNCTION_IS_ENABLED = 26;
		private static byte FUNCTION_SET_DECAY = 27;
		private static byte FUNCTION_GET_DECAY = 28;
		private static byte FUNCTION_SET_MINIMUM_VOLTAGE = 29;
		private static byte FUNCTION_GET_MINIMUM_VOLTAGE = 30;
		private static byte CALLBACK_UNDER_VOLTAGE = 31;
		private static byte CALLBACK_POSITION_REACHED = 32;
		private static byte FUNCTION_SET_SYNC_RECT = 33;
		private static byte FUNCTION_IS_SYNC_RECT = 34;
		private static byte FUNCTION_SET_TIME_BASE = 35;
		private static byte FUNCTION_GET_TIME_BASE = 36;
		private static byte FUNCTION_GET_ALL_DATA = 37;
		private static byte FUNCTION_SET_ALL_DATA_PERIOD = 38;
		private static byte FUNCTION_GET_ALL_DATA_PERIOD = 39;
		private static byte CALLBACK_ALL_DATA = 40;
		private static byte CALLBACK_NEW_STATE = 41;
		private static byte FUNCTION_RESET = 243;
		private static byte FUNCTION_GET_CHIP_TEMPERATURE = 242;


		/// <summary>
		///  This callback is triggered when the input voltage drops below the value set by
		///  <see cref="Tinkerforge.BrickStepper.SetMinimumVoltage"/>. The parameter is the current voltage given
		///  in mV.
		/// </summary>
		public delegate void UnderVoltage(ushort voltage);

		/// <summary>
		///  This callback is triggered when a position set by <see cref="Tinkerforge.BrickStepper.SetSteps"/> or
		///  <see cref="Tinkerforge.BrickStepper.SetTargetPosition"/> is reached.
		///  
		///  <note>
		///   Since we can't get any feedback from the stepper motor, this only works if the
		///   acceleration (see <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/>) is set smaller or equal to the
		///   maximum acceleration of the motor. Otherwise the motor will lag behind the
		///   control value and the callback will be triggered too early.
		///  </note>
		/// </summary>
		public delegate void PositionReached(int position);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickStepper.SetAllDataPeriod"/>. The parameters are: the current velocity,
		///  the current position, the remaining steps, the stack voltage, the external
		///  voltage and the current consumption of the stepper motor.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public delegate void AllData(ushort currentVelocity, int currentPosition, int remainingSteps, ushort stackVoltage, ushort externalVoltage, ushort currentConsumption);

		/// <summary>
		///  This callback is triggered whenever the Stepper Brick enters a new state. 
		///  It returns the new state as well as the previous state.
		///  
		///  Possible states are:
		///  
		///  * Stop = 1
		///  * Acceleration = 2
		///  * Run = 3
		///  * Deacceleration = 4
		///  * Direction change to forward = 5
		///  * Direction change to backward = 6
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public delegate void NewState(byte stateNew, byte statePrevious);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickStepper(string uid) : base(uid) 
		{
			this.expectedName = "Stepper Brick";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 2;
			messageCallbacks[CALLBACK_UNDER_VOLTAGE] = new MessageCallback(CallbackUnderVoltage);
			messageCallbacks[CALLBACK_POSITION_REACHED] = new MessageCallback(CallbackPositionReached);
			messageCallbacks[CALLBACK_ALL_DATA] = new MessageCallback(CallbackAllData);
			messageCallbacks[CALLBACK_NEW_STATE] = new MessageCallback(CallbackNewState);
		}

		/// <summary>
		///  Sets the maximum velocity of the stepper motor in steps per second.
		///  This function does *not* start the motor, it merely sets the maximum
		///  velocity the stepper motor is accelerated to. To get the motor running use
		///  either <see cref="Tinkerforge.BrickStepper.SetTargetPosition"/>, <see cref="Tinkerforge.BrickStepper.SetSteps"/>, <see cref="Tinkerforge.BrickStepper.DriveForward"/> or
		///  <see cref="Tinkerforge.BrickStepper.DriveBackward"/>.
		/// </summary>
		public void SetMaxVelocity(ushort velocity)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MAX_VELOCITY, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(velocity, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the velocity as set by <see cref="Tinkerforge.BrickStepper.SetMaxVelocity"/>.
		/// </summary>
		public ushort GetMaxVelocity()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MAX_VELOCITY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MAX_VELOCITY, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Returns the *current* velocity of the stepper motor in steps per second.
		/// </summary>
		public ushort GetCurrentVelocity()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_VELOCITY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_VELOCITY, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Sets the acceleration and deacceleration of the stepper motor. The values
		///  are given in *steps/s²*. An acceleration of 1000 means, that
		///  every second the velocity is increased by 1000 *steps/s*.
		///  
		///  For example: If the current velocity is 0 and you want to accelerate to a
		///  velocity of 8000 *steps/s* in 10 seconds, you should set an acceleration
		///  of 800 *steps/s²*.
		///  
		///  An acceleration/deacceleration of 0 means instantaneous
		///  acceleration/deacceleration (not recommended)
		///  
		///  The default value is 1000 for both
		/// </summary>
		public void SetSpeedRamping(ushort acceleration, ushort deacceleration)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_SPEED_RAMPING, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(acceleration, 4, data_);
			LEConverter.To(deacceleration, 6, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the acceleration and deacceleration as set by 
		///  <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/>.
		/// </summary>
		public void GetSpeedRamping(out ushort acceleration, out ushort deacceleration)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_SPEED_RAMPING, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_SPEED_RAMPING, out response);

			acceleration = LEConverter.UShortFrom(4, response);
			deacceleration = LEConverter.UShortFrom(6, response);
		}

		/// <summary>
		///  Executes an active full brake. 
		///   
		///  <note type="caution">
		///   This function is for emergency purposes,
		///   where an immediate brake is necessary. Depending on the current velocity and
		///   the strength of the motor, a full brake can be quite violent.
		///  </note>
		///  
		///  Call <see cref="Tinkerforge.BrickStepper.Stop"/> if you just want to stop the motor.
		/// </summary>
		public void FullBrake()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_FULL_BRAKE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Sets the current steps of the internal step counter. This can be used to
		///  set the current position to 0 when some kind of starting position
		///  is reached (e.g. when a CNC machine reaches a corner).
		/// </summary>
		public void SetCurrentPosition(int position)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CURRENT_POSITION, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(position, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the current position of the stepper motor in steps. On startup
		///  the position is 0. The steps are counted with all possible driving
		///  functions (<see cref="Tinkerforge.BrickStepper.SetTargetPosition"/>, <see cref="Tinkerforge.BrickStepper.SetSteps"/>, <see cref="Tinkerforge.BrickStepper.DriveForward"/> or
		///  <see cref="Tinkerforge.BrickStepper.DriveBackward"/>). It also is possible to reset the steps to 0 or
		///  set them to any other desired value with <see cref="Tinkerforge.BrickStepper.SetCurrentPosition"/>.
		/// </summary>
		public int GetCurrentPosition()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_POSITION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_POSITION, out response);

			return LEConverter.IntFrom(4, response);
		}

		/// <summary>
		///  Sets the target position of the stepper motor in steps. For example,
		///  if the current position of the motor is 500 and <see cref="Tinkerforge.BrickStepper.SetTargetPosition"/> is
		///  called with 1000, the stepper motor will drive 500 steps forward. It will
		///  use the velocity, acceleration and deacceleration as set by
		///  <see cref="Tinkerforge.BrickStepper.SetMaxVelocity"/> and <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/>.
		///  
		///  A call of <see cref="Tinkerforge.BrickStepper.SetTargetPosition"/> with the parameter *x* is equivalent to
		///  a call of <see cref="Tinkerforge.BrickStepper.SetSteps"/> with the parameter 
		///  (*x* - <see cref="Tinkerforge.BrickStepper.GetCurrentPosition"/>).
		/// </summary>
		public void SetTargetPosition(int position)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_TARGET_POSITION, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(position, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the last target position as set by <see cref="Tinkerforge.BrickStepper.SetTargetPosition"/>.
		/// </summary>
		public int GetTargetPosition()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_TARGET_POSITION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_TARGET_POSITION, out response);

			return LEConverter.IntFrom(4, response);
		}

		/// <summary>
		///  Sets the number of steps the stepper motor should run. Positive values
		///  will drive the motor forward and negative values backward. 
		///  The velocity, acceleration and deacceleration as set by
		///  <see cref="Tinkerforge.BrickStepper.SetMaxVelocity"/> and <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/> will be used.
		/// </summary>
		public void SetSteps(int steps)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_STEPS, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(steps, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the last steps as set by <see cref="Tinkerforge.BrickStepper.SetSteps"/>.
		/// </summary>
		public int GetSteps()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STEPS, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STEPS, out response);

			return LEConverter.IntFrom(4, response);
		}

		/// <summary>
		///  Returns the remaining steps of the last call of <see cref="Tinkerforge.BrickStepper.SetSteps"/>.
		///  For example, if <see cref="Tinkerforge.BrickStepper.SetSteps"/> is called with 2000 and 
		///  <see cref="Tinkerforge.BrickStepper.GetRemainingSteps"/> is called after the motor has run for 500 steps,
		///  it will return 1500.
		/// </summary>
		public int GetRemainingSteps()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_REMAINING_STEPS, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_REMAINING_STEPS, out response);

			return LEConverter.IntFrom(4, response);
		}

		/// <summary>
		///  Sets the step mode of the stepper motor. Possible values are:
		///  
		///  * Full Step = 1
		///  * Half Step = 2
		///  * Quarter Step = 4
		///  * Eighth Step = 8
		///  
		///  A higher value will increase the resolution and
		///  decrease the torque of the stepper motor.
		///  
		///  The default value is 8 (Eighth Step).
		/// </summary>
		public void SetStepMode(byte mode)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_STEP_MODE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(mode, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the step mode as set by <see cref="Tinkerforge.BrickStepper.SetStepMode"/>.
		/// </summary>
		public byte GetStepMode()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STEP_MODE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STEP_MODE, out response);

			return LEConverter.ByteFrom(4, response);
		}

		/// <summary>
		///  Drives the stepper motor forward until <see cref="Tinkerforge.BrickStepper.DriveBackward"/> or
		///  <see cref="Tinkerforge.BrickStepper.Stop"/> is called. The velocity, acceleration and deacceleration as 
		///  set by <see cref="Tinkerforge.BrickStepper.SetMaxVelocity"/> and <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/> will be used.
		/// </summary>
		public void DriveForward()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_DRIVE_FORWARD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Drives the stepper motor backward until <see cref="Tinkerforge.BrickStepper.DriveForward"/> or
		///  <see cref="Tinkerforge.BrickStepper.Stop"/> is triggered. The velocity, acceleration and deacceleration as
		///  set by <see cref="Tinkerforge.BrickStepper.SetMaxVelocity"/> and <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/> will be used.
		/// </summary>
		public void DriveBackward()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_DRIVE_BACKWARD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Stops the stepper motor with the deacceleration as set by 
		///  <see cref="Tinkerforge.BrickStepper.SetSpeedRamping"/>.
		/// </summary>
		public void Stop()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_STOP, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the stack input voltage in mV. The stack input voltage is the
		///  voltage that is supplied via the stack, i.e. it is given by a 
		///  Step-Down or Step-Up Power Supply.
		/// </summary>
		public ushort GetStackInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STACK_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STACK_INPUT_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Returns the external input voltage in mV. The external input voltage is
		///  given via the black power input connector on the Stepper Brick. 
		///   
		///  If there is an external input voltage and a stack input voltage, the motor
		///  will be driven by the external input voltage. If there is only a stack 
		///  voltage present, the motor will be driven by this voltage.
		///  
		///  <note type="caution">
		///   This means, if you have a high stack voltage and a low external voltage,
		///   the motor will be driven with the low external voltage. If you then remove
		///   the external connection, it will immediately be driven by the high
		///   stack voltage
		///  </note>
		/// </summary>
		public ushort GetExternalInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Returns the current consumption of the motor in mA.
		/// </summary>
		public ushort GetCurrentConsumption()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_CONSUMPTION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_CONSUMPTION, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Sets the current in mA with which the motor will be driven.
		///  The minimum value is 100mA, the maximum value 2291mA and the 
		///  default value is 800mA.
		///  
		///  <note type="caution">
		///   Do not set this value above the specifications of your stepper motor.
		///   Otherwise it may damage your motor.
		///  </note>
		/// </summary>
		public void SetMotorCurrent(ushort current)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MOTOR_CURRENT, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(current, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the current as set by <see cref="Tinkerforge.BrickStepper.SetMotorCurrent"/>.
		/// </summary>
		public ushort GetMotorCurrent()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MOTOR_CURRENT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MOTOR_CURRENT, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Enables the driver chip. The driver parameters can be configured (maximum velocity,
		///  acceleration, etc) before it is enabled.
		/// </summary>
		public void Enable()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_ENABLE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Disables the driver chip. The configurations are kept (maximum velocity,
		///  acceleration, etc) but the motor is not driven until it is enabled again.
		/// </summary>
		public void Disable()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_DISABLE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns *true* if the driver chip is enabled, *false* otherwise.
		/// </summary>
		public bool IsEnabled()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_ENABLED, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_IS_ENABLED, out response);

			return LEConverter.BoolFrom(4, response);
		}

		/// <summary>
		///  Sets the decay mode of the stepper motor. The possible value range is
		///  between 0 and 65535. A value of 0 sets the fast decay mode, a value of
		///  65535 sets the slow decay mode and a value in between sets the mixed
		///  decay mode.
		///  
		///  Changing the decay mode is only possible if synchronous rectification
		///  is enabled (see <see cref="Tinkerforge.BrickStepper.SetSyncRect"/>).
		///  
		///  For a good explanation of the different decay modes see 
		///  `this &lt;http://ebldc.com/?p=86/&gt;`__ blog post by Avayan.
		///  
		///  A good decay mode is unfortunately different for every motor. The best
		///  way to work out a good decay mode for your stepper motor, if you can't
		///  measure the current with an oscilloscope, is to listen to the sound of
		///  the motor. If the value is too low, you often hear a high pitched 
		///  sound and if it is too high you can often hear a humming sound.
		///  
		///  Generally, fast decay mode (small value) will be noisier but also
		///  allow higher motor speeds.
		///  
		///  The default value is 10000.
		///  
		///  <note>
		///   There is unfortunately no formula to calculate a perfect decay
		///   mode for a given stepper motor. If you have problems with loud noises
		///   or the maximum motor speed is too slow, you should try to tinker with
		///   the decay value
		///  </note>
		/// </summary>
		public void SetDecay(ushort decay)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DECAY, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(decay, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the decay mode as set by <see cref="Tinkerforge.BrickStepper.SetDecay"/>.
		/// </summary>
		public ushort GetDecay()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DECAY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_DECAY, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Sets the minimum voltage in mV, below which the <see cref="Tinkerforge.BrickStepper.UnderVoltage"/> callback
		///  is triggered. The minimum possible value that works with the Stepper Brick is 8V.
		///  You can use this function to detect the discharge of a battery that is used
		///  to drive the stepper motor. If you have a fixed power supply, you likely do 
		///  not need this functionality.
		///  
		///  The default value is 8V.
		/// </summary>
		public void SetMinimumVoltage(ushort voltage)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(voltage, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the minimum voltage as set by <see cref="Tinkerforge.BrickStepper.SetMinimumVoltage"/>.
		/// </summary>
		public ushort GetMinimumVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MINIMUM_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Turns synchronous rectification on or off (true/false).
		///  
		///  With synchronous rectification on, the decay can be changed
		///  (see <see cref="Tinkerforge.BrickStepper.SetDecay"/>). Without synchronous rectification fast
		///  decay is used.
		///  
		///  For an explanation of synchronous rectification see 
		///  `here &lt;http://en.wikipedia.org/wiki/Active_rectification&gt;`__.
		///  
		///  <note type="caution">
		///   If you want to use high speeds (&gt; 10000 steps/s) for a large 
		///   stepper motor with a large inductivity we strongly
		///   suggest that you disable synchronous rectification. Otherwise the
		///   Brick may not be able to cope with the load and overheat.
		///  </note>
		///  
		///  The default value is *false*.
		/// </summary>
		public void SetSyncRect(bool syncRect)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_SYNC_RECT, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(syncRect, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns *true* if synchronous rectification is enabled, *false* otherwise.
		/// </summary>
		public bool IsSyncRect()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_SYNC_RECT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_IS_SYNC_RECT, out response);

			return LEConverter.BoolFrom(4, response);
		}

		/// <summary>
		///  Sets the time base of the velocity and the acceleration of the stepper brick
		///  (in seconds).
		///  
		///  For example, if you want to make one step every 1.5 seconds, you can set 
		///  the time base to 15 and the velocity to 10. Now the velocity is 
		///  10steps/15s = 1steps/1.5s.
		///  
		///  The default value is 1.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public void SetTimeBase(uint timeBase)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_TIME_BASE, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(timeBase, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the time base as set by <see cref="Tinkerforge.BrickStepper.SetTimeBase"/>.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public uint GetTimeBase()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_TIME_BASE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_TIME_BASE, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Returns the following parameters: The current velocity,
		///  the current position, the remaining steps, the stack voltage, the external
		///  voltage and the current consumption of the stepper motor.
		///  
		///  There is also a callback for this function, see <see cref="Tinkerforge.BrickStepper.AllData"/>.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public void GetAllData(out ushort currentVelocity, out int currentPosition, out int remainingSteps, out ushort stackVoltage, out ushort externalVoltage, out ushort currentConsumption)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ALL_DATA, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_ALL_DATA, out response);

			currentVelocity = LEConverter.UShortFrom(4, response);
			currentPosition = LEConverter.IntFrom(6, response);
			remainingSteps = LEConverter.IntFrom(10, response);
			stackVoltage = LEConverter.UShortFrom(14, response);
			externalVoltage = LEConverter.UShortFrom(16, response);
			currentConsumption = LEConverter.UShortFrom(18, response);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickStepper.AllData"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public void SetAllDataPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_ALL_DATA_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickStepper.SetAllDataPeriod"/>.
		///  
		///  .. versionadded:: 1.1.6
		/// </summary>
		public uint GetAllDataPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ALL_DATA_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_ALL_DATA_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Calling this function will reset the Brick. Calling this function
		///  on a Brick inside of a stack will reset the whole stack.
		///  
		///  After a reset you have to create new device objects,
		///  calling functions on the existing ones will result in
		///  undefined behavior!
		/// </summary>
		public void Reset()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_RESET, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the temperature in °C/10 as measured inside the microcontroller. The
		///  value returned is not the ambient temperature!
		///  
		///  The temperature is only proportional to the real temperature and it has an
		///  accuracy of +-15%. Practically it is only useful as an indicator for
		///  temperature changes.
		/// </summary>
		public short GetChipTemperature()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIP_TEMPERATURE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CHIP_TEMPERATURE, out response);

			return LEConverter.ShortFrom(4, response);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMaxVelocity(out ushort velocity)
		{
			velocity = GetMaxVelocity();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentVelocity(out ushort velocity)
		{
			velocity = GetCurrentVelocity();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentPosition(out int position)
		{
			position = GetCurrentPosition();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetTargetPosition(out int position)
		{
			position = GetTargetPosition();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetSteps(out int steps)
		{
			steps = GetSteps();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetRemainingSteps(out int steps)
		{
			steps = GetRemainingSteps();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStepMode(out byte mode)
		{
			mode = GetStepMode();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStackInputVoltage(out ushort voltage)
		{
			voltage = GetStackInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetExternalInputVoltage(out ushort voltage)
		{
			voltage = GetExternalInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentConsumption(out ushort current)
		{
			current = GetCurrentConsumption();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMotorCurrent(out ushort current)
		{
			current = GetMotorCurrent();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsEnabled(out bool enabled)
		{
			enabled = IsEnabled();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDecay(out ushort decay)
		{
			decay = GetDecay();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMinimumVoltage(out ushort voltage)
		{
			voltage = GetMinimumVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsSyncRect(out bool syncRect)
		{
			syncRect = IsSyncRect();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetTimeBase(out uint timeBase)
		{
			timeBase = GetTimeBase();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAllDataPeriod(out uint period)
		{
			period = GetAllDataPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChipTemperature(out short temperature)
		{
			temperature = GetChipTemperature();
		}

		internal int CallbackUnderVoltage(byte[] data_)
		{
			ushort voltage = LEConverter.UShortFrom(4, data_);

			((UnderVoltage)callbacks[CALLBACK_UNDER_VOLTAGE])(voltage);
			return 6;
		}

		internal int CallbackPositionReached(byte[] data_)
		{
			int position = LEConverter.IntFrom(4, data_);

			((PositionReached)callbacks[CALLBACK_POSITION_REACHED])(position);
			return 8;
		}

		internal int CallbackAllData(byte[] data_)
		{
			ushort currentVelocity = LEConverter.UShortFrom(4, data_);
			int currentPosition = LEConverter.IntFrom(6, data_);
			int remainingSteps = LEConverter.IntFrom(10, data_);
			ushort stackVoltage = LEConverter.UShortFrom(14, data_);
			ushort externalVoltage = LEConverter.UShortFrom(16, data_);
			ushort currentConsumption = LEConverter.UShortFrom(18, data_);

			((AllData)callbacks[CALLBACK_ALL_DATA])(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption);
			return 20;
		}

		internal int CallbackNewState(byte[] data_)
		{
			byte stateNew = LEConverter.ByteFrom(4, data_);
			byte statePrevious = LEConverter.ByteFrom(5, data_);

			((NewState)callbacks[CALLBACK_NEW_STATE])(stateNew, statePrevious);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(UnderVoltage))
			{
				callbacks[CALLBACK_UNDER_VOLTAGE] = d;
			}
			else if(d.GetType() == typeof(PositionReached))
			{
				callbacks[CALLBACK_POSITION_REACHED] = d;
			}
			else if(d.GetType() == typeof(AllData))
			{
				callbacks[CALLBACK_ALL_DATA] = d;
			}
			else if(d.GetType() == typeof(NewState))
			{
				callbacks[CALLBACK_NEW_STATE] = d;
			}
		}
	}
}
